/*
 * file Provider.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import java.util.Map;

import javax.wvcm.ProviderFactory.Callback;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A WVCM provider.
 *
 * A WVCM provider is a factory for creating new locations, resource lists
 * and the generic Resource proxy. 
 * 
 * @since 1.0
 */
public interface Provider {
	
    /**
     * Initialize the provider with the specified map and callback.
     * This is used to initialize a provider that was created with the zero-argument constructor.
     * @param initArgs arguments used to customize the behavior of the provider.
     * @param callback allows the provider to call back to the client for additional information.
     * @throws WvcmException if the initArgs are invalid.
     */
	public void initialize(Map<String, String> initArgs, Callback callback) throws WvcmException;
    
    /**
     * Return the initialization arguments for this provider.
     * @return the initialization arguments for this provider.
     */
    public Map<String, String> initArgs();
        
    /**
     * Return the initialization arguments for this provider.
     * @return the initialization arguments for this provider.
     */
    public Callback callback();
    
    /**
     * A valid folder location for this provider.
     * 
     * @return a valid folder location for this provider.
     * If this provider supports access to multiple repositories,
     * each sibling (i.e. resource with the same parent) of this folder
     * will commonly identify a repository.
     */
    public Location rootLocation();
        
    /**
     * A location used in a property of a resource to indicate that this location should 
     * be interpreted relative to the folder that contains the resource.
     */
    public Location relativeRootLocation();
    
    /**
     * Location factory for this Provider.
     * 
     * @param string the format of the location string is specific to the Provider,
     * and to the repository that stores the persistent resource.
     * A URL, a UNC filename, and an NFS filename
     * are examples of possible formats for a location string.
     * 
     * @return a Location whose string value is the specified String.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#ILLEGAL_LOCATION_SYNTAX}:
     *  String is not a valid location.
     */
    public Location location(String string) throws WvcmException;

    /**
     * Construct a proxy of class T for the given location.
     * 
     * @param type the type of the proxy.
     * @param loc the location of the resource.
     * @return a new proxy for a resource at this Location.
     */
    public <T extends Resource> T buildProxy(Class<T> type, Location loc)
    throws WvcmException; 
    
    /**
     * Construct a resource proxy for the given location.
     * 
     * @param loc the location of the resource.
     * @return a new proxy for a resource at this Location.
     */
    public Resource resource(Location loc);

    /**
     * Construct a folder proxy for the given location.
     * 
     * @param loc the location of the folder.
     * @return a new proxy for a folder at this Location.
     */
    public Folder folder(Location loc);

    /**
     * Construct a new {@link ResourceList}.
     * 
     * @param resources An array of resource proxies from which the list
     * is to be initialized. 
     * 
     * @return a ResourceList initialized by the specified resources.
     */
    public <T extends Resource> ResourceList<T> resourceList(T... resources);
    
}
